--
-- Sonic Adventure-esque monitor display
-- (c) Inuyasha 2014
-- Permission is granted to use this code in your own mods.
-- This code should be future-proof and work with all custom monitors.
--

local QUESTIONBOX_DETECT = 296396

local function draw_monitor(v, stplyr)
	if not stplyr.monitor_list or stplyr.monitor_time <= 0
		return
	end

	-- Slowly scale down the icons when time is nearly up.
	local scale = FRACUNIT
	if stplyr.monitor_time < TICRATE
		scale = stplyr.monitor_time<<FRACBITS / TICRATE
	end

	for i, monitor in ipairs(stplyr.monitor_list)
		-- Get the sprite name and cache it
		local sprite = v.cachePatch(monitor.spr)
		v.drawScaled(monitor.x<<FRACBITS, 184<<FRACBITS, scale, sprite, V_SNAPTOBOTTOM)
	end
end
hud.add(draw_monitor);

--
-- MonitorPop drop-in replacement
--
function A_MonitorPop(actor, var1, var2)
-- Everything below here is Adventure Items
	if not actor.target or not actor.target.player
		-- Make sure to call the super function!
		super(actor, var1, var2)
		return
	end

	-- This is a bit fun.
	-- Get the target player and sprite name
	-- before MonitorPop is done, since they're not valid
	-- after it's called.
	local player = actor.target.player
	local sstate

	-- Question box hack
	local realscale = actor.destscale
	if actor.type != MT_MYSTERY_BOX
		sstate = states[mobjinfo[actor.info.damage].spawnstate]
	else
		sstate = QUESTIONBOX_DETECT
		actor.destscale = QUESTIONBOX_DETECT
	end

	-- Call old monitor pop
	super(actor, var1, var2)

	-- Question box hack
	-- This is unfortunately stupid, but required.
	-- MonitorPop doesn't give us any info about what was spawned.
	if sstate == QUESTIONBOX_DETECT
		for mobj in thinkers.iterate("mobj")
			if mobj.destscale != QUESTIONBOX_DETECT
				continue
			end
			-- Remains
			mobj.destscale = realscale
			if not (mobj.flags & MF_BOXICON)
				continue
			end
			sstate = states[mobj.info.spawnstate]
		end
	end

	local sprite = string.format("%s%c0", sprnames[sstate.sprite], string.byte('A')+sstate.frame)

	-- 1up hack (too lazy to do overlays)
	if sprite == "PRUPD0"
		sprite = "PRUPB0"
	end

	-- Init if it doesn't exist
	if not player.monitor_list
		player.monitor_list = {}
	end

	-- Remove oldest if we're out of room
	if #player.monitor_list > 8
		table.remove(player.monitor_list, 1)
	end

	-- And now insert our new monitor into the table!
	table.insert(player.monitor_list, {x = 0, spr = sprite})
	player.monitor_time = TICRATE*5/2
	
	-- Sonic 1 Dragon Edition Monitors - Air Bubbles
	if DragonMonitorsModule()
		P_SpawnMobj(actor.x, actor.y, actor.z, MT_EXTRALARGEBUBBLE)
	end
end

-- Adventure Items but for golden monitors
function A_GoldMonitorPop(actor, var1, var2)
	if not actor.target or not actor.target.player
		-- Make sure to call the super function!
		super(actor, var1, var2)
		return
	end

	-- This is a bit fun.
	-- Get the target player and sprite name
	-- before MonitorPop is done, since they're not valid
	-- after it's called.
	local player = actor.target.player
	local sstate

	-- Question box hack
	local realscale = actor.destscale
	if actor.type != MT_MYSTERY_BOX
		sstate = states[mobjinfo[actor.info.damage].spawnstate]
	else
		sstate = QUESTIONBOX_DETECT
		actor.destscale = QUESTIONBOX_DETECT
	end

	-- Call old monitor pop
	super(actor, var1, var2)

	-- Question box hack
	-- This is unfortunately stupid, but required.
	-- MonitorPop doesn't give us any info about what was spawned.
	if sstate == QUESTIONBOX_DETECT
		for mobj in thinkers.iterate("mobj")
			if mobj.destscale != QUESTIONBOX_DETECT
				continue
			end
			-- Remains
			mobj.destscale = realscale
			if not (mobj.flags & MF_BOXICON)
				continue
			end
			sstate = states[mobj.info.spawnstate]
		end
	end

	local sprite = string.format("%s%c0", sprnames[sstate.sprite], string.byte('A')+sstate.frame)

	-- 1up hack (too lazy to do overlays)
	if sprite == "PRUPD0"
		sprite = "PRUPB0"
	end

	-- Init if it doesn't exist
	if not player.monitor_list
		player.monitor_list = {}
	end

	-- Remove oldest if we're out of room
	if #player.monitor_list > 8
		table.remove(player.monitor_list, 1)
	end

	-- And now insert our new monitor into the table!
	table.insert(player.monitor_list, {x = 0, spr = sprite})
	player.monitor_time = TICRATE*5/2
end

--
-- Handling player's monitors
-- (HUD code can do no modifying, so)
--
local function handle_monitors(player)
	-- Initialization if not already set
	if player.monitor_time == nil
		player.monitor_time = 0
	end

	-- Decrement monitor time if it is set.
	-- If not then just remove the list of monitors.
	if player.monitor_list and player.monitor_time > 0
		player.monitor_time = $1 - 1
	else
		player.monitor_list = nil
		return
	end

	-- And for any monitor pairs we have..
	for i, monitor in ipairs(player.monitor_list)
		-- Get the final X position first;
		-- the final position should have all elements perfectly center
		local finalxpos = 160 + ((#player.monitor_list-1) * 16) - ((i-1) * 32)

		-- Now move the monitor icon on screen ... at least 8 pixels
		monitor.x = $1 + max(8, (finalxpos - monitor.x) / 6)

		-- And if it's past the destination, just put it back there.
		if monitor.x > finalxpos
			monitor.x = finalxpos
		end
	end
end

--
-- Think hook
--
local function hook_think()
	for player in players.iterate()
		handle_monitors(player)
	end
end
addHook("ThinkFrame", hook_think)
